@@ -1,8 +1,8 @@
 # csvw - https://w3c.github.io/csvw/primer/
 
 from .metadata import (
-    TableGroup, Table, Column, ForeignKey, Link, NaturalLanguage, Datatype, URITemplate,
-)
+    TableGroup, Table, Column, ForeignKey, Link, NaturalLanguage, Datatype, URITemplate, CSVW,
+    Dialect)
 
 from .dsv import (UnicodeWriter,
     UnicodeReader, UnicodeReaderWithLineNumber, UnicodeDictReader, NamedTupleReader,
@@ -14,13 +14,14 @@ __all__ = [
     'Link', 'NaturalLanguage',
     'Datatype',
     'URITemplate',
-    'UnicodeWriter',
+    'Dialect', 'UnicodeWriter',
     'UnicodeReader', 'UnicodeReaderWithLineNumber', 'UnicodeDictReader', 'NamedTupleReader',
     'iterrows', 'rewrite',
+    'CSVW',
 ]
 
 __title__ = 'csvw'
-__version__ = '2.0.0'
+__version__ = '3.0.0'
 __author__ = 'Robert Forkel'
 __license__ = 'Apache 2.0, see LICENSE'
 __copyright__ = 'Copyright (c) 2022 Robert Forkel'
