@@ -2,9 +2,10 @@
 SQLite as alternative storage backend for a TableGroup's data.
 
 For the most part, translation of a TableGroup's tableSchema to SQL works as expected:
-- each table is converted to a "CREATE TABLE" statement
-- each column specifies a column in the corresponding "CREATE TABLE" statement
-- foreignKey constraints are added according to the corresponding tableSchema property.
+
+- each table is converted to a `CREATE TABLE` statement
+- each column specifies a column in the corresponding `CREATE TABLE` statement
+- `foreignKey` constraints are added according to the corresponding `tableSchema` property.
 
 List-valued foreignKeys are supported as follows: For each pair of tables related through a
 list-valued foreign key, an association table is created. To make it possible to distinguish
@@ -13,14 +14,16 @@ a column `context`, which stores the name of the foreign key column from which a
 assocation table was created.
 
 SQL table and column names can be customized by passing a translator callable when instantiating
-a `Database`.
+a :class:`Database`.
 
 SQLite support has the following limitations:
+
 - lists as values (as specified via the separator attribute of a Column) are only supported for
   string types.
-- regex constraints on strings (as specified via a Datatype's format attribute) are not enforced
-  by the database.
+- regex constraints on strings (as specified via a :class:`csvw.Datatype`'s format attribute) are
+  not enforced by the database.
 """
+import typing
 import decimal
 import pathlib
 import sqlite3
@@ -30,7 +33,9 @@ import collections
 
 import attr
 
+import csvw
 from csvw.datatypes import DATATYPES
+from csvw.metadata import TableGroup
 
 
 def identity(s):
@@ -92,7 +97,7 @@ def select(db, table):
 @attr.s
 class ColSpec(object):
     """
-    A `ColSpec` captures sufficient information about a `Column` for the DB schema.
+    A `ColSpec` captures sufficient information about a :class:`csvw.Column` for the DB schema.
     """
     name = attr.ib()
     csvw_type = attr.ib(default='string', converter=lambda s: s if s else 'string')
@@ -148,7 +153,7 @@ class ColSpec(object):
                 constraints.append('length(`{0}`) <= {1}'.format(cname, c.maxLength))
         return ' AND '.join(constraints)
 
-    def sql(self, translate):
+    def sql(self, translate) -> str:
         _check = self.check(translate)
         return '`{0}` {1}{2}{3}'.format(
             translate(self.name),
@@ -160,14 +165,16 @@ class ColSpec(object):
 @attr.s
 class TableSpec(object):
     """
-    A `TableSpec` captures sufficient information about a `Table` for the DB schema.
+    A `TableSpec` captures sufficient information about a :class:`csvw.Table` for the DB schema.
 
-    Note: We support "light-weight" many-to-many relationships by allowing list-valued foreign key
-    columns in CSVW. In the database these columns are turned into an associative table, adding
-    the name of the column as value a "context" column. Thus, multiple columns in a table my be
-    specified as targets of many-to-many relations with the same table.
+    .. note::
 
-    See also https://en.wikipedia.org/wiki/Associative_entity
+        We support "light-weight" many-to-many relationships by allowing list-valued foreign key
+        columns in CSVW. In the database these columns are turned into an associative table, adding
+        the name of the column as value a `context` column. Thus, multiple columns in a table my be
+        specified as targets of many-to-many relations with the same table.
+
+        .. seealso:: `<https://en.wikipedia.org/wiki/Associative_entity>`_
     """
     name = attr.ib()
     columns = attr.ib(default=attr.Factory(list))
@@ -176,7 +183,7 @@ class TableSpec(object):
     primary_key = attr.ib(default=None)
 
     @classmethod
-    def from_table_metadata(cls, table):
+    def from_table_metadata(cls, table: csvw.Table) -> 'TableSpec':
         """
         Create a `TableSpec` from the schema description of a `csvw.metadata.Table`.
 
@@ -222,7 +229,7 @@ class TableSpec(object):
         return spec
 
     @classmethod
-    def association_table(cls, atable, apk, btable, bpk):
+    def association_table(cls, atable, apk, btable, bpk) -> 'TableSpec':
         afk = ColSpec('{0}_{1}'.format(atable, apk))
         bfk = ColSpec('{0}_{1}'.format(btable, bpk))
         if afk.name == bfk.name:
@@ -237,7 +244,7 @@ class TableSpec(object):
             ]
         )
 
-    def sql(self, translate):
+    def sql(self, translate) -> str:
         """
         :param translate:
         :return: The SQL statement to create the table.
@@ -256,7 +263,7 @@ class TableSpec(object):
             translate(self.name), ',\n    '.join(clauses))
 
 
-def schema(tg):
+def schema(tg: csvw.TableGroup) -> typing.List[TableSpec]:
     """
     Convert the table and column descriptions of a `TableGroup` into specifications for the
     DB schema.
@@ -291,7 +298,14 @@ def schema(tg):
 
 
 class Database(object):
-    def __init__(self, tg, fname=None, translate=None):
+    """
+    Represents a SQLite database associated with a :class:`csvw.TableGroup` instance.
+    """
+    def __init__(
+            self,
+            tg: TableGroup,
+            fname: typing.Optional[typing.Union[pathlib.Path, str]] = None,
+            translate: typing.Optional[typing.Callable] = None):
         self.translate = translate or Database.name_translator
         self.fname = pathlib.Path(fname) if fname else None
         self.init_schema(tg)
@@ -302,11 +316,11 @@ class Database(object):
         self.tables = schema(self.tg) if self.tg else []
 
     @property
-    def tdict(self):
+    def tdict(self) -> typing.Dict[str, TableSpec]:
         return {t.name: t for t in self.tables}
 
     @staticmethod
-    def name_translator(table, column=None):
+    def name_translator(table: str, column: typing.Optional[str] = None) -> str:
         """
         A callable with this signature can be passed into DB creation to control the names
         of the schema objects.
@@ -318,7 +332,7 @@ class Database(object):
         # By default, no translation is done:
         return column or table
 
-    def connection(self):
+    def connection(self) -> typing.Union[sqlite3.Connection, contextlib.closing]:
         if self.fname:
             return contextlib.closing(sqlite3.connect(str(self.fname)))
         if not self._connection:
@@ -342,13 +356,27 @@ FROM {2} {3} GROUP BY {0}""".format(
             r[0]: [(k, v) if context is None else k
                    for k, v in zip(r[1].split(), r[2].split('||'))] for r in cu.fetchall()}
 
-    def read(self):
+    def separator(self, tname: str, cname: str) -> typing.Optional[str]:
+        """
+        :return: separator for the column specified by db schema names `tname` and `cname`.
+        """
+        for name in self.tdict:
+            if self.translate(name) == tname:
+                for col in self.tdict[name].columns:
+                    if self.translate(name, col.name) == cname:
+                        return col.separator
+
+    def split_value(self, tname, cname, value) -> typing.Union[typing.List[str], str, None]:
+        sep = self.separator(tname, cname)
+        return (value or '').split(sep) if sep else value
+
+    def read(self) -> typing.Dict[str, typing.List[typing.OrderedDict]]:
         res = collections.defaultdict(list)
         with self.connection() as conn:
             for tname in self.tg.tabledict:
                 #
                 # FIXME: how much do we want to use DB types? Probably as much as possible!
-                # Thus we need tp convert on write **and** read!
+                # Thus we need to convert on write **and** read!
                 #
                 convert, seps, refs = {}, {}, collections.defaultdict(dict)
                 table = self.tdict[tname]  # The TableSpec object.
