@@ -15,6 +15,7 @@ import io
 import csv
 import codecs
 import shutil
+import typing
 import pathlib
 import tempfile
 import warnings
@@ -36,9 +37,29 @@ def normalize_encoding(encoding):
 
 
 class UnicodeWriter(object):
-    """Write Unicode data to a csv file."""
+    """
+    Write Unicode data to a csv file.
 
-    def __init__(self, f=None, dialect=None, **kw):
+    :param f: The target to which to write the data; a local path specified as `str` or \
+    `pathlib.Path` or `None`, in which case the data, formatted as DSV can be retrieved \
+    via :meth:`~UnicodeWriter.read`
+    :param dialect: Either a dialect name as recognized by `csv.writer` or a \
+    :class:`~Dialect` instance for dialect customization beyond what can be done with \
+    `csv.writer`.
+    :param kw: Keyword arguments passed through to `csv.writer`.
+
+    .. code-block:: python
+
+        >>> from csvw import UnicodeWriter
+        >>> with UnicodeWriter('data.tsv', delimiter='\t') as writer:
+        ...     writer.writerow(['ä', 'ö', 'ü'])
+    """
+
+    def __init__(
+            self,
+            f: typing.Optional[typing.Union[str, pathlib.Path]] = None,
+            dialect: typing.Optional[typing.Union[Dialect, str]] = None,
+            **kw):
         self.f = f
         self.encoding = kw.pop('encoding', 'utf-8')
         if isinstance(dialect, Dialect):
@@ -78,7 +99,11 @@ class UnicodeWriter(object):
         self.writer = csv.writer(self.f, **self.kw)
         return self
 
-    def read(self):
+    def read(self) -> typing.Optional[bytes]:
+        """
+        If the writer has been initialized passing `None` as target, the CSV data as `bytes` can be
+        retrieved calling this method.
+        """
         if hasattr(self.f, 'seek'):
             self.f.seek(0)
         if hasattr(self.f, 'read'):
@@ -88,18 +113,40 @@ class UnicodeWriter(object):
         if self._close:
             self.f.close()
 
-    def writerow(self, row):
+    def writerow(self, row: typing.Union[tuple, list]):
         self.writer.writerow(self._escapedoubled(row))
 
-    def writerows(self, rows):
+    def writerows(self, rows: typing.Iterable[typing.Union[tuple, list]]):
         for row in rows:
             self.writerow(row)
 
 
 class UnicodeReader(object):
-    """Read Unicode data from a csv file."""
+    """
+    Read Unicode data from a csv file.
 
-    def __init__(self, f, dialect=None, **kw):
+    :param f: The source from which to read the data; a local path specified as `str` or \
+    `pathlib.Path`, a file-like object or a `list` of lines.
+    :param dialect: Either a dialect name as recognized by `csv.reader` or a \
+    :class:`~Dialect` instance for dialect customization beyond what can be done with \
+    `csv.writer`.
+    :param kw: Keyword arguments passed through to `csv.reader`.
+
+    .. code-block:: python
+
+        >>> with UnicodeReader('tests/fixtures/frictionless-data.csv', delimiter='|') as reader:
+        ...     for row in reader:
+        ...         print(row)
+        ...         break
+        ...
+        ['FK', 'Year', 'Location name', 'Value', 'binary', 'anyURI', 'email', 'boolean', 'array',
+        'geojson']
+    """
+    def __init__(
+            self,
+            f: typing.Union[str, pathlib.Path, typing.IO, typing.List[str]],
+            dialect: typing.Optional[typing.Union[Dialect, str]] = None,
+            **kw):
         self.f = f
         self.encoding = normalize_encoding(kw.pop('encoding', 'utf-8-sig'))
         self.newline = kw.pop('lineterminator', None)
@@ -158,10 +205,13 @@ class UnicodeReader(object):
                    row[0].startswith(self.dialect.commentPrefix)) or \
                     ((not row or set(row) == {''}) and self.dialect.skipBlankRows) or \
                     (self.lineno < self.dialect.skipRows):
-                if row and self.dialect.commentPrefix and \
-                        row[0].startswith(self.dialect.commentPrefix):
-                    self.comments.append(
-                        (self.lineno, self.dialect.delimiter.join(row)[1:].strip()))
+                if (row and self.dialect.commentPrefix and  # noqa: W504
+                        row[0].startswith(self.dialect.commentPrefix)) or \
+                        (row and self.lineno < self.dialect.skipRows):
+                    self.comments.append((
+                        self.lineno,
+                        self.dialect.delimiter.join(row).lstrip(self.dialect.commentPrefix).strip(),
+                    ))
                 row = self._next_row()
             row = [self.dialect.trimmer(s) for s in row][self.dialect.skipColumns:]
         return row
@@ -175,10 +225,12 @@ class UnicodeReader(object):
 
 
 class UnicodeReaderWithLineNumber(UnicodeReader):
-
+    """
+    A `UnicodeReader` yielding (lineno, row) pairs, where "lineno" is the 1-based number of the
+    the **text line** where the (possibly multi-line) row data starts in the DSV file.
+    """
     def __next__(self):
         """
-
         :return: a pair (1-based line number in the input, row)
         """
         # Retrieve the row, thereby incrementing the line number:
@@ -187,7 +239,27 @@ class UnicodeReaderWithLineNumber(UnicodeReader):
 
 
 class UnicodeDictReader(UnicodeReader):
-    """Read Unicode data represented as one (ordered) dictionary per row."""
+    """
+    A `UnicodeReader` yielding one `dict` per row.
+
+    :param f: As for :class:`UnicodeReader`
+    :param fieldnames:
+
+    .. code-block:: python
+
+        >>> with UnicodeDictReader(
+        ...         'tests/fixtures/frictionless-data.csv',
+        ...         dialect=Dialect(delimiter='|', header=False),
+        ...         fieldnames=[str(i) for i in range(1, 11)]) as reader:
+        ...     for row in reader:
+        ...         print(row)
+        ...         break
+        ...
+        OrderedDict([('1', 'FK'), ('2', 'Year'), ('3', 'Location name'), ('4', 'Value'),
+        ('5', 'binary'), ('6', 'anyURI'), ('7', 'email'), ('8', 'boolean'), ('9', 'array'),
+        ('10', 'geojson')])
+
+    """
 
     def __init__(self, f, fieldnames=None, restkey=None, restval=None, **kw):
         self._fieldnames = fieldnames   # list of keys for the dict
@@ -236,7 +308,14 @@ class UnicodeDictReader(UnicodeReader):
 
 
 class NamedTupleReader(UnicodeDictReader):
-    """Read namedtuple objects from a csv file."""
+    """
+    A `UnicodeReader` yielding one `namedtuple` per row.
+
+    .. note::
+
+        This reader has some limitations, notably that fieldnames must be normalized to be
+        admissible Python names, but also bad performance (compared with `UnicodeDictReader`).
+    """
 
     _normalize_fieldname = staticmethod(utils.normalize_name)
 
@@ -282,7 +361,7 @@ reader = iterrows
 
 
 def rewrite(fname, visitor, **kw):
-    """Utility function to rewrite rows in tsv files.
+    """Utility function to rewrite rows in dsv files.
 
     :param fname: Path of the dsv file to operate on.
     :param visitor: A callable that takes a line-number and a row as input and returns a \
